#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time

from colored.controls import Controls


def main() -> None:
    """Test control movements.
    """
    moves: dict[str, int] = {
        'down': 10,
        'up': 7,
        'right': 20,
        'left': 5,
        'previous': 10,
        'horizontal': 5,
        'erase_display': 2,
        'erase_line': 5,
        'scroll_up': 3,
        'scroll_down': 10
    }

    control = Controls()
    for move, row in moves.items():
        print(f'{move} {control.nav(move, row)}')
        time.sleep(1)

    print(f"position 4, 5 {control.nav('position', 4, 5)}")


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time

from colored.hexadecimal import Hex


def main():
    """Test the shorthand #abc hex expansion.
    Contribution by Fredrik Klasson. """

    hexad = Hex()
    all_ok = True
    try:
        T_sta = time.perf_counter()
        for r in range(0, 0xF):
            for g in range(0, 0xF):
                for b in range(0, 0xF):
                    short = '#%x%x%x' % (r, g, b)
                    long = '#%x%x%x%x%x%x' % (r, r, g, g, b, b)
                    all_ok = all_ok and hexad.find(long) == hexad.find(short)
                print('.', end='')
            print('')

        dT = time.perf_counter() - T_sta
        print('Lookup time: %0.4f s => %0.4f s/lookup' % (dT, dT / (2 * 0xF * 0xF * 0xF)))
        print('pass' if all_ok else 'FAILED')
    except Exception as e:
        print('Whopsie, something %s-ish went wrong: %s' % (e.__class__.__name__, e))
        import traceback
        traceback.print_exc()
        all_ok = False

    return all_ok


if __name__ == "__main__":
    ok = main()
    exit(0 if ok else 1)



import unittest

from colored.background import Back
from colored.colored import Colored, back, back_rgb
from colored.library import Library

# pylint: disable=invalid-name


class TestBackground(unittest.TestCase):

    """Test for background colors names and codes.
    """

    def setUp(self) -> None:
        self.ESC: str = '\x1b['
        self.BACKGROUND_256: str = f'{self.ESC}48;5;'
        self.BACKGROUND_RGB: str = f'{self.ESC}48;2;'
        self.END: str = 'm'

    def test_background_256_code(self) -> None:
        """Test backgrounds color by code.
        """
        for code in Library.COLORS.values():
            color = Colored(code)
            self.assertEqual(f'{self.BACKGROUND_256}{code}{self.END}', color.background())

    def test_background_256_name(self) -> None:
        """Test background by name.
        """
        for name, code in Library.COLORS.items():
            color = Colored(name)
            self.assertEqual(f'{self.BACKGROUND_256}{code}{self.END}', color.background())

    def test_back_256_func(self) -> None:
        """Test function backgrounds color by code.
        """
        for name, code in Library.COLORS.items():
            color = Colored(name)
            self.assertEqual(back(code), color.background())

    def test_background_rgb(self) -> None:
        """Test rgb background colors.
        """
        for code in Library.COLORS.values():
            self.assertEqual(f'{self.BACKGROUND_RGB}{code};{code};{code}{self.END}', Back.rgb(code, code, code))

    def test_background_rgb_func(self) -> None:
        """Test function rgb background colors.
        """
        for code in Library.COLORS.values():
            self.assertEqual(f'{self.BACKGROUND_RGB}{code};{code};{code}{self.END}', back_rgb(code, code, code))


if __name__ == '__main__':
    unittest.main()



from colored.background import Back
from colored.colored import Colored, back, back_rgb
from colored.library import Library

ESC: str = '\x1b['
BACKGROUND_256: str = f'{ESC}48;5;'
BACKGROUND_RGB: str = f'{ESC}48;2;'
END: str = 'm'


def test_background_256_code() -> None:
    for code in Library.COLORS.values():
        color = Colored(code)
        assert f'{BACKGROUND_256}{code}{END}' == color.background()


def test_background_256_name() -> None:
    for name, code in Library.COLORS.items():
        color = Colored(name)
        assert f'{BACKGROUND_256}{code}{END}' == color.background()


def test_back_256_func() -> None:
    for name, code in Library.COLORS.items():
        color = Colored(name)
        assert back(code) == color.background()


def test_background_rgb() -> None:
    for code in Library.COLORS.values():
        assert f'{BACKGROUND_RGB}{code};{code};{code}{END}' == Back.rgb(code, code, code)


def test_background_rgb_func() -> None:
    for code in Library.COLORS.values():
        assert f'{BACKGROUND_RGB}{code};{code};{code}{END}' == back_rgb(code, code, code)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

from typing import NoReturn

from .exceptions import InvalidStyle
from .library import Library
from .utilities import Utilities


class MetaStyle(type):
    """Metaclass to customize attribute access."""
    def __getattr__(cls, color: str) -> NoReturn:
        """Override __getattr__ to raise InvalidColor when an invalid style is accessed."""
        raise InvalidStyle(f'InvalidStyle: {color}')


class Style(metaclass=MetaStyle):

    """ Style class contains predefined styles for formatting
    text in the command line (terminal).
    """

    _utils = Utilities()
    _ESC: str = Library.ESC
    _END: str = Library.END
    _STYLES: dict[str, str] = Library.STYLES
    _UNDER: str = Library.UNDERLINE_COLOR
    _COLORS: dict[str, str] = Library.COLORS

    for _style, _code in _STYLES.items():
        vars()[_style] = f'{_ESC}{_code}{_END}'
        vars()[_style.upper()] = f'{_ESC}{_code}{_END}'

    @classmethod
    def underline_color(cls: type[Style], color: str | int) -> str:
        """Combination with text returns color underline text.
        print(f"{Style.underline_color('red')}White text with red color underline.")

        Args:
            color (str | int): Sets the name of the underline color,
                   numbers are also acceptable.

        Returns:
            str: Underline color code.
        """
        color = str(color).lower()
        cls._utils.is_color_exist(color)
        if not color.isdigit():
            color = cls._COLORS[color]
        return f'{cls._UNDER}{color}{cls._END}'

    @classmethod
    def UNDERLINE_COLOR(cls: type[Style], color: str | int) -> str:  # pylint: disable=[C0103]
        """Combination with text returns color underline text.
        print(f"{Style.underline_color('red')}White text with red color underline.")

        Args:
            color (str | int): Sets the name of the underline color,
                         numbers are also acceptable.

        Returns:
            str: Underline color code.
        """
        return cls.underline_color(color)


class style(Style):  # pylint: disable=[C0103]
    """ This will be deprecated in the future, do not use this for version >= 2.0.0,
        instead please use Style class (See issue #28). """



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

import os

from .exceptions import (InvalidColor, InvalidControl, InvalidHexColor,
                         InvalidStyle)
from .library import Library

# pylint: disable=invalid-name


class Utilities:

    """ Class Utilities that managing some behaviors.
    """

    def __init__(self) -> None:
        self._COLORS: dict[str, str] = Library.COLORS
        self._HEX_COLORS: dict[str, str] = Library.HEX_COLORS
        self._STYLES: dict[str, str] = Library.STYLES
        self._CONTROLS: dict[str, str] = Library.CONTROLS
        self._COLORTERM: dict[str, int] = Library.COLORTERM
        self.RGB_MAXIMUM_COLOR: int = 255
        self.colorterm: str = 'truecolor'
        self.set_colorterm()

    def is_color_exist(self, name: str) -> bool:
        """Checks for valid color by name or by hex style name or number,
            and raise a InvalidColor or InvalidHexColor exception if it doesn't exist.

        Args:
            name (str): Sets the name of the style or the number.

        Returns:
            bool: True if existed.

        Raises:
            InvalidColor: Invalid color message.
        """
        if name.startswith('#'):
            if name not in self._HEX_COLORS.values():
                raise InvalidHexColor(f'InvalidHexColor: {name}')

        elif name not in self._COLORS and name not in self._COLORS.values():
            raise InvalidColor(f'InvalidColor: {name}')

        return True

    def is_style_exist(self, name: str) -> bool:
        """Checks for valid style and raise a InvalidStyle exception
            if it doesn't exist.

        Args:
            name (str): Sets the name of the style or the number.

        Returns:
            bool: True if exists.

        Raises:
            InvalidStyle: Invalid style message.
        """
        if name not in self._STYLES and name not in self._STYLES.values():
            raise InvalidStyle(f'InvalidStyle: {name}')

        return True

    def is_control_exist(self, name: str) -> bool:
        """Checks for valid control and raise a InvalidControl exception
            if it doesn't exist.

        Args:
            name (str): Sets the name of the control.

        Returns:
            bool: True if exists.

        Raises:
            InvalidControl: Invalid control message.
        """
        if name.lower() not in self._CONTROLS:
            raise InvalidControl(f'InvalidControl: {name}')

        return True

    def convert_percentages(self, percent: str) -> int:
        """Convert percentages to color number with range 0-255:

        Args:
            percent (str): Sets the percent number (e.g. '50%').

        Returns:
            int: Color number in range [0, 255]. Returns 0 for unrecognized values.
        """
        try:  # Sets maximum range for RGB colors.
            self.RGB_MAXIMUM_COLOR = self._COLORTERM[self.colorterm]
        except KeyError:
            pass

        percentages: dict[str, int] = {}
        for number in range(self.RGB_MAXIMUM_COLOR + 1):
            percentage = int((number / self.RGB_MAXIMUM_COLOR) * 100)
            percentages[f'{percentage}%'] = number
        try:
            color = percentages[percent]
        except KeyError:
            color = 0

        return color

    def set_colorterm(self, colorterm: str = '') -> None:
        """Set the  environment variable if it's necessary.
         used to indicate whether a terminal emulator
        could use colors, or whether it supports 8-bit color or more.

        Args:
            colorterm (str, optional): Sets the  environment variable.
        """
        if not colorterm:
            self.colorterm = os.getenv('COLORTERM') or self.colorterm
        else:
            self.colorterm = colorterm
            os.environ['COLORTERM'] = colorterm

    def is_percentage(self, numbers: tuple[int | str, int | str, int | str]) -> tuple[int, int, int]:
        """Checks a tuple of RGB numbers and convert them to integers.

        Args:
            numbers (tuple): Sets percentages of numbers.

        Returns:
            tuple: Tuple with RGB numbers (all integers after conversion).
        """
        r_in, g_in, b_in = numbers

        r_out = self.convert_percentages(str(r_in)) if isinstance(r_in, str) and r_in.endswith('%') else int(r_in)
        g_out = self.convert_percentages(str(g_in)) if isinstance(g_in, str) and g_in.endswith('%') else int(g_in)
        b_out = self.convert_percentages(str(b_in)) if isinstance(b_in, str) and b_in.endswith('%') else int(b_in)

        return (max(0, min(255, r_out)), max(0, min(255, g_out)), max(0, min(255, b_out)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

from typing import NoReturn

from .exceptions import InvalidColor
from .library import Library
from .utilities import Utilities


class MetaBack(type):
    """Metaclass to customize attribute access."""
    def __getattr__(cls, color: str) -> NoReturn:
        """Override __getattr__ to raise InvalidColor when an invalid color is accessed."""
        raise InvalidColor(f'InvalidColor: {color}')


class Back(metaclass=MetaBack):

    """ Back class contains predefined background colors for
    the command line (terminal).
    """

    _utils = Utilities()
    _END: str = Library.END
    _COLORS: dict[str, str] = Library.COLORS
    _BACKGROUND_256: str = Library.BACKGROUND_256
    _BACKGROUND_RGB: str = Library.BACKGROUND_RGB

    for _color, _code in _COLORS.items():
        vars()[_color] = f'{_BACKGROUND_256}{_code}{_END}'
        vars()[_color.upper()] = f'{_BACKGROUND_256}{_code}{_END}'

    @classmethod
    def rgb(cls: type[Back], r: int | str, g: int | str, b: int | str) -> str:
        """Combination with text returns color background with text.

        Args:
            r (int | str): Sets the Red color.
            g (int | str): Sets the Green color.
            b (int | str): Sets the Blue color.

        Returns:
            str: Background RGB code.
        """
        r, g, b = cls._utils.is_percentage((r, g, b))
        return f'{cls._BACKGROUND_RGB}{r};{g};{b}{cls._END}'

    @classmethod
    def RGB(cls: type[Back], r: int | str, g: int | str, b: int | str) -> str:  # pylint: disable=[C0103]
        """Combination with text returns color background with text.

        Args:
            r (int | str): Sets the Red color.
            g (int | str): Sets the Green color.
            b (int | str): Sets the Blue color.

        Returns:
            str: Background RGB code.
        """
        return cls.rgb(r, g, b)


class back(Back):  # pylint: disable=[C0103]
    """This will be deprecated in the future, do not use this for version >= 2.0.0,
        instead please use Back class (See issue #28)."""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from typing import Union

from .colored import fore
from .library import Library


class Convert:

    """Using colored libraries to match HEX code to ANSI code and opposite.
    """

    def hex_to_ansi(self, hex_code: str) -> Union[str, None]:
        """Convert HEX code to ANSI code string.

        Args:
            hex_code (str): HEX code string.

        Returns:
            Union[str, None]: ANSI code number.
        """
        if hex_code in Library.HEX_COLORS.values():
            key: str = self._get_key_from_value(hex_code)
            return fore(key)
        return None

    def ansi_to_hex(self, ansi_code: int) -> Union[str, None]:
        """Convert ANSI code to HEX code string.

        Args:
            ansi_code (int): ANSI number code.

        Returns:
            Union[str, None]: HEX code string.
        """
        if str(ansi_code) in Library.HEX_COLORS:
            return Library.HEX_COLORS[str(ansi_code)]
        return None

    def _get_key_from_value(self, value: str) -> str:
        """Return the key of the dictionary with value.

        Args:
            value (str): Dictionary value.

        Returns:
            (str): The key of the dictionary.
        """
        for key, val in Library.HEX_COLORS.items():
            if val == value:
                return key
        return ''



import unittest

from colored.colored import Colored, fore, fore_rgb
from colored.foreground import Fore
from colored.library import Library

# pylint: disable=invalid-name


class TestForeground(unittest.TestCase):

    """Test for foreground colors names and codes.
    """

    def setUp(self) -> None:
        self.ESC: str = '\x1b['
        self.FOREGROUND_256: str = f'{self.ESC}38;5;'
        self.FOREGROUND_RGB: str = f'{self.ESC}38;2;'
        self.END: str = 'm'

    def test_foreground_256_code(self) -> None:
        """Test foreground color by code.
        """
        for code in Library.COLORS.values():
            color = Colored(code)
            self.assertEqual(f'{self.FOREGROUND_256}{code}{self.END}', color.foreground())

    def test_foreground_256_name(self) -> None:
        """Test foreground color by name.
        """
        for name, code in Library.COLORS.items():
            color = Colored(name)
            self.assertEqual(f'{self.FOREGROUND_256}{code}{self.END}', color.foreground())

    def test_fore_256_func(self) -> None:
        """Test function foregrounds color by code.
        """
        for name, code in Library.COLORS.items():
            color = Colored(name)
            self.assertEqual(fore(code), color.foreground())

    def test_foreground_rgb(self) -> None:
        """Test function foregrounds color by name.
        """
        for code in Library.COLORS.values():
            self.assertEqual(f'{self.FOREGROUND_RGB}{code};{code};{code}{self.END}', Fore.rgb(code, code, code))

    def test_foreground_rgb_func(self) -> None:
        """Test function rgb foreground colors.
        """
        for code in Library.COLORS.values():
            self.assertEqual(f'{self.FOREGROUND_RGB}{code};{code};{code}{self.END}', fore_rgb(code, code, code))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from colored import back, fore, style
from colored.library import Library


def main() -> None:
    """Test foreground and background colors.
    """

    for name in Library.HEX_COLORS.values():
        print(f"{fore(name)}{name:>3}", end='  ')

    for name in Library.HEX_COLORS.values():
        print(f"{back(name)}{name:>3}", end='  ')

    print(style('reset'))  # Reset all attributes


if __name__ == '__main__':
    main()



from colored.convert import Convert
from colored.library import Library


def test_hex_to_ansi() -> None:
    convert = Convert()
    for hex_code in Library.HEX_COLORS.values():
        result = convert.hex_to_ansi(hex_code)
        assert result is not None
        # Result is an ANSI escape sequence — extract the code number
        ansi_num = result.replace('\x1b[38;5;', '').replace('m', '')
        assert Library.HEX_COLORS.get(ansi_num) == hex_code


def test_ansi_to_hex() -> None:
    convert = Convert()
    for ansi_code, hex_code in Library.HEX_COLORS.items():
        result = convert.ansi_to_hex(int(ansi_code))
        assert result == hex_code


def test_hex_to_ansi_invalid() -> None:
    convert = Convert()
    assert convert.hex_to_ansi('#000000invalid') is None


def test_ansi_to_hex_invalid() -> None:
    convert = Convert()
    assert convert.ansi_to_hex(9999) is None



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from colored import back, fore, style


def main() -> None:
    """Test foreground and background colors with int numbers.
    """

    for num in range(0, 256):
        print(f"{fore(num)}{num:>3}{style(0)}", end='  ')

    for num in range(0, 256):
        print(f"{back(num)}{num:>3}{style(0)}", end='  ')

    for name in range(0, 256):
        print(f"{style(4, name)}{name:>3}{style(59)}", end='  ')


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from colored import back, fore, style
from colored.library import Library


def main() -> None:
    """Test foreground and background colors.
    """

    # Prints foreground colors
    for name in Library.COLORS:
        print(f"{fore(name)}{name:>3}{style('reset')}", end='  ')

    # Prints background colors
    for name in Library.COLORS:
        print(f"{style(4, name)}{back(name)}{name:>3}{style('reset')}", end='  ')

    # Prints underline with colors
    for name in Library.COLORS:
        print(f"{style(4, name)}{name:>3}{style(59)}", end='  ')


if __name__ == '__main__':
    main()



import pytest

from colored.exceptions import InvalidHexColor
from colored.hexadecimal import Hex
from colored.library import Library

HEX_NEAREST: dict[str, tuple[str, ...]] = {
    '1':  ('#7f0000', '#800000', '#810000'),
    '2':  ('#007f00', '#008000', '#008100'),
    '4':  ('#00007f', '#000080', '#000081'),
    '10': ('#00fe00', '#00ff00', '#01ff00'),
    '12': ('#0000fe', '#0000ff', '#0100ff'),
    '14': ('#00feff', '#00ffff', '#00fffe'),
}


@pytest.mark.parametrize("expected,hex_codes", HEX_NEAREST.items())
def test_hex_nearest_match(expected: str, hex_codes: tuple[str, ...]) -> None:
    hexad = Hex()
    for hex_code in hex_codes:
        nearest = hexad.find(hex_code)
        assert Library.HEX_COLORS[nearest] == Library.HEX_COLORS[expected]


def test_hex_invalid_format_raises() -> None:
    hexad = Hex()
    for invalid in ('#FF', '#AABBCCDD', 'red', ''):
        with pytest.raises(InvalidHexColor):
            hexad.find(invalid)


def test_hex_shorthand_expansion() -> None:
    hexad = Hex()
    for r in range(0, 0xF):
        for g in range(0, 0xF):
            for b in range(0, 0xF):
                short = '#%x%x%x' % (r, g, b)
                long = '#%x%x%x%x%x%x' % (r, r, g, g, b, b)
                assert hexad.find(long) == hexad.find(short)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest

from colored.exceptions import (InvalidColor, InvalidControl, InvalidHexColor,
                                InvalidStyle)


class TestInvalid(unittest.TestCase):

    """Test for the Exceptions.
    """

    def test_exception_raised_invalidcolor(self) -> None:
        """Test that the exception is raised correctly."""
        with self.assertRaises(InvalidColor) as context:
            raise InvalidColor("InvalidColor: red_90")

        # Check that the error message is correct.
        self.assertEqual(str(context.exception), "InvalidColor: red_90")

    def test_exception_message_invalidcolor(self) -> None:
        """Test the exception message directly."""
        exception = InvalidColor("InvalidColor: red_90")
        self.assertEqual(str(exception), "InvalidColor: red_90")

    def test_exception_inheritance_invalidcolor(self) -> None:
        """Test that InvalidColor inherits from Exception."""
        exception = InvalidColor("Test message")
        self.assertIsInstance(exception, Exception)

    def test_exception_raised_invalidhexcolor(self) -> None:
        """Test that the exception is raised correctly."""
        with self.assertRaises(InvalidHexColor) as context:
            raise InvalidHexColor("InvalidHexColor: #005fdj")

        # Check that the error message is correct.
        self.assertEqual(str(context.exception), "InvalidHexColor: #005fdj")

    def test_exception_message_invalidhexcolor(self) -> None:
        """Test the exception message directly."""
        exception = InvalidHexColor("InvalidHexColor: blue_15")
        self.assertEqual(str(exception), "InvalidHexColor: blue_15")

    def test_exception_inheritance_invalidhexcolor(self) -> None:
        """Test that InvalidHexColor inherits from Exception."""
        exception = InvalidHexColor("Test message")
        self.assertIsInstance(exception, Exception)

    def test_exception_raised_invalidcontrol(self) -> None:
        """Test that the exception is raised correctly."""
        with self.assertRaises(InvalidControl) as context:
            raise InvalidControl("InvalidControl: upper")

        # Check that the error message is correct.
        self.assertEqual(str(context.exception), "InvalidControl: upper")

    def test_exception_message_invalidcontrol(self) -> None:
        """Test the exception message directly."""
        exception = InvalidControl("InvalidControl: righting")
        self.assertEqual(str(exception), "InvalidControl: righting")

    def test_exception_inheritance_invalidcontrol(self) -> None:
        """Test that InvalidControl inherits from Exception."""
        exception = InvalidControl("Test message")
        self.assertIsInstance(exception, Exception)

    def test_exception_raised_invalidstyle(self) -> None:
        """Test that the exception is raised correctly."""
        with self.assertRaises(InvalidStyle) as context:
            raise InvalidStyle("InvalidStyle: bolb")

        # Check that the error message is correct.
        self.assertEqual(str(context.exception), "InvalidStyle: bolb")

    def test_exception_message_invalidstyle(self) -> None:
        """Test the exception message directly."""
        exception = InvalidStyle("InvalidStyle: bolb")
        self.assertEqual(str(exception), "InvalidStyle: bolb")

    def test_exception_inheritance_invalidstyle(self) -> None:
        """Test that InvalidStyle inherits from Exception."""
        exception = InvalidStyle("Test message")
        self.assertIsInstance(exception, Exception)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import random

from colored.cprint import cprint
from colored.library import Library


def main() -> None:
    """Test cprint(text: str,
    fore_256: int | str = '',
    back_256: int | str = '',
    fore_rgb: tuple = (255, 255, 255),
    back_rgb: tuple = (0, 0, 0),
    formatting: int | str = '',
    line_color: int | str = '',
    reset=True,
    **kwargs) function.
    """

    for color in Library.COLORS.values():
        back: int = random.randint(0, 255)
        a: int = random.randint(0, 255)
        b: int = random.randint(0, 255)
        c: int = random.randint(0, 255)
        style: str = random.choice(list(Library.STYLES.values()))
        cprint(' Hello World or World Hello? ', fore_256=color, back_256=back, formatting=style)
        cprint(' Hello World or World Hello? ', fore_rgb=(a, b, c), back_rgb=(c, b, a), formatting=style)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Tests foreground and background RGB colors with numbers percentages. """

import time

from colored import Back, Fore, Style
from colored.controls import Controls
from colored.utilities import Utilities


def foreground() -> None:
    """Tests foreground and background RGB colors with numbers percentages.
    """

    n = 0
    control = Controls()
    utils = Utilities()
    print("Press '[CTRL] + C' to interrupt:\n")
    print(f'{"":>7}FOREGROUND {"":>9}COLOR {"":>8}BACKGROUND')
    try:
        while True:
            n += 1
            per: str = f'{n}%'
            code: int = utils.convert_percentages(per)
            print(f'\r[{Fore.rgb(code, 0, 0)} Red {Style.reset}]'
                  f'[{Fore.rgb(0, code, 0)} Green {Style.reset}]'
                  f'[{Fore.rgb(0, 0, code)} Blue {Style.reset}] [ {per:>4} ] '
                  f'[{Back.rgb(code, 0, 0)} Red {Style.reset}]'
                  f'[{Back.rgb(0, code, 0)} Green {Style.reset}]'
                  f'[{Back.rgb(0, 0, code)} Blue {Style.reset}]',
                  end='')
            if n == 100:
                n = 0
                print(control.nav('erase_line', 1), end='')
                time.sleep(1)
            time.sleep(0.01)
    except KeyboardInterrupt:
        pass


def main() -> None:
    """Main function.
    """
    foreground()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

import os
import platform
import sys
from typing import Optional

from .hexadecimal import Hex
from .library import Library
from .utilities import Utilities

# TTY_AWARE = True
# IS_TTY = sys.stdout.isatty() and sys.stderr.isatty()
# _win_vterm_mode = None

# pylint: disable=invalid-name


class Config:  # pylint: disable=[R0903]

    """ Config some global variables.

    Attributes:
        TTY_AWARE (bool): If True, colorization will be disabled when stdout/stderr
                          are not connected to a TTY (terminal).
    """

    TTY_AWARE: bool = True
    WIN_VTERM_MODE: Optional[bool] = None

    @staticmethod
    def is_tty() -> bool:
        """ Check if both sys.stdout and sys.stderr are connected to a TTY (terminal).

        This function determines whether the script is running in an interactive
        terminal or if the output has been redirected (e.g., to a file or pipe).

        Returns:
            bool: True if both stdout and stderr are attached to a TTY, False otherwise.

        Example:
            >>> is_tty()
            True  # When running in a terminal

            >>> python script.py > output.txt
            False  # When stdout is redirected to a file
        """
        return sys.stdout.isatty() and sys.stderr.isatty()


class Colored:  # pylint: disable=[R0902]

    """ Colored class, it's the main class of managing
    foreground, background and style colors.
    """

    def __init__(self, name: str | int) -> None:
        """name can be str or int instead.

        Args:
            name (Union[str, int]): name of color or number of color
        """
        self._name: str = str(name).lower()
        self._hex_color: str = ''
        self._hex = Hex()
        self._utils = Utilities()

        # self._ESC: str = Library.ESC
        # self._END: str = Library.END

        # self._STYLES: dict[str, str] = Library.STYLES
        # self._FOREGROUND_256: str = Library.FOREGROUND_256
        # self._BACKGROUND_256: str = Library.BACKGROUND_256

        # self._COLORS: dict[str, str] = Library.COLORS
        # self._HEX_COLORS: dict[str, str] = Library.HEX_COLORS
        # self._UNDERLINE_COLOR: str = Library.UNDERLINE_COLOR

        if self._name.startswith('#'):
            self._hex_color = self._hex.find(self._name)

    def attribute(self, line_color: str | int = '') -> str:
        """ Returns stylize text.

        Args:
            line_color: Sets color of the underline.

        Returns:
            str: Style code.
        """
        formatting: str = self._name
        if not self.enabled():
            return ''

        if self._name:
            self._utils.is_style_exist(self._name)

            if self._name in ('underline', '4') and line_color:
                line_color = str(line_color).lower()
                self._utils.is_color_exist(line_color)
                if not line_color.isdigit():
                    line_color = Library.COLORS[line_color]
                return f'{Library.UNDERLINE_COLOR}{line_color}{Library.END}'

            if not self._name.isdigit():
                formatting = Library.STYLES[self._name]

        return f'{Library.ESC}{formatting}{Library.END}'

    def foreground(self) -> str:
        """ Returns a foreground 256 color code. """
        color: str = self._name
        if not self.enabled():
            return ''

        if self._name:
            self._utils.is_color_exist(self._name)

            if self._name.startswith('#'):
                color = self._hex_color
            elif not self._name.isdigit():
                color = Library.COLORS[self._name]

        return f'{Library.FOREGROUND_256}{color}{Library.END}'

    def background(self) -> str:
        """ Returns a background 256 color code. """
        color: str = self._name
        if not self.enabled():
            return ''

        if self._name:
            self._utils.is_color_exist(self._name)

            if self._name.startswith('#'):
                color = self._hex_color
            elif not self._name.isdigit():
                color = Library.COLORS[self._name]

        return f'{Library.BACKGROUND_256}{color}{Library.END}'

    @staticmethod
    def enable_windows_terminal_mode() -> Optional[bool]:
        """Contribution by: Andreas Fredrik Klasson, Magnus Heskestad,
        Dimitris Papadopoulos.

        Enable virtual terminal processing in Windows terminal. Does
        nothing if not on Windows. This is based on the rejected
        enhancement <https://bugs.python.org/issue29059>.

        Returns:
            Optional[bool]: True if successful, False if an error occurred or not on Windows,
                            or the previously determined state if already run.
        """
        if Config.WIN_VTERM_MODE is not None:
            return Config.WIN_VTERM_MODE

        Config.WIN_VTERM_MODE = platform.system().lower() == 'windows'

        if Config.WIN_VTERM_MODE is False:
            return Config.WIN_VTERM_MODE

        try:
            # pylint: disable=[C0415]
            from ctypes import (byref, c_void_p,  # type: ignore[attr-defined]
                                windll, wintypes)
            ENABLE_VIRTUAL_TERMINAL_PROCESSING = 0x0004
            INVALID_HANDLE_VALUE = c_void_p(-1).value
            STD_OUTPUT_HANDLE = wintypes.DWORD(-11)

            hStdout = windll.kernel32.GetStdHandle(STD_OUTPUT_HANDLE)
            if hStdout == INVALID_HANDLE_VALUE:
                Config.WIN_VTERM_MODE = False
            else:
                mode = wintypes.DWORD(0)
                ok = windll.kernel32.GetConsoleMode(wintypes.HANDLE(hStdout), byref(mode))
                if not ok:
                    Config.WIN_VTERM_MODE = False
                else:
                    mode = wintypes.DWORD(mode.value | ENABLE_VIRTUAL_TERMINAL_PROCESSING)
                    ok = windll.kernel32.SetConsoleMode(wintypes.HANDLE(hStdout), mode)
                    if not ok:
                        Config.WIN_VTERM_MODE = False
        except ImportError:
            Config.WIN_VTERM_MODE = False
        except Exception:  # pylint: disable=[W0718]
            Config.WIN_VTERM_MODE = False

        return Config.WIN_VTERM_MODE

    @staticmethod
    def enabled() -> bool:
        """ Contribution by Andreas Motl.
        https://github.com/chalk/supports-color#info
        Use the environment variable FORCE_COLOR=1 (level 1), FORCE_COLOR=2
        (level 2), or FORCE_COLOR=3 (level 3) to forcefully enable color, or
        FORCE_COLOR=0 to forcefully disable. The use of FORCE_COLOR overrides
        all other color support checks.
        """
        if 'FORCE_COLOR' in os.environ:
            try:
                if int(os.environ['FORCE_COLOR']) == 0:
                    return False
            except ValueError:
                pass
            return True

        # https://no-color.org/
        # Check for the presence of a NO_COLOR environment variable that, when
        # present (regardless of its value), prevents the addition of ANSI
        # color.
        if 'NO_COLOR' in os.environ:
            return False

        # Also disable coloring when not printing to a TTY.
        if Config.TTY_AWARE and not Config.is_tty():
            return False

        # In all other cases, enable coloring.
        return True


def style(name: int | str, color: str | int = '') -> str:
    """ Alias for Colored(name).attribute()

    Args:
        name: Sets the name of the color.
        color: Sets the underline color.

    Returns:
        str: Style code.
    """
    return Colored(name).attribute(color)


def fore(name: int | str) -> str:
    """ Combination with text returns color text.

    Args:
        name: Sets the name of the color.

    Returns:
        str: Foreground code.
    """
    return Colored(name).foreground()


def back(name: int | str) -> str:
    """ Combination with text returns color background with text.

    Args:
        name: Sets the name of the color.

    Returns:
        str: Background code.
    """
    return Colored(name).background()


def fore_rgb(r: int | str, g: int | str, b: int | str) -> str:
    """ Combination with text returns color text.

    Args:
        r: Red color.
        g: Green color.
        b: Blue color.

    Returns:
        str: Foreground RGB code.
    """
    utils = Utilities()
    r, g, b = utils.is_percentage((r, g, b))
    return f'{Library.FOREGROUND_RGB}{r};{g};{b}{Library.END}'


def back_rgb(r: int | str, g: int | str, b: int | str) -> str:
    """ Combination with text returns color background with text.

    Args:
        r: Red color.
        g: Green color.
        b: Blue color.

    Returns:
        str: Background RGB code.
    """
    utils = Utilities()
    r, g, b = utils.is_percentage((r, g, b))
    return f'{Library.BACKGROUND_RGB}{r};{g};{b}{Library.END}'


def attr(name: int | str) -> str:
    """ This will be deprecated in the future, do not use with version >= 2.0.0,
    instead please use style() function (See issue #28).

    Args:
        name: Sets the name of the color.

    Returns:
        str: Style code.
    """
    return Colored(name).attribute()


def fg(name: int | str) -> str:
    """ This will be deprecated in the future, do not use with version >= 2.0.0,
    instead please use style() function (See issue #28).

    Args:
        name: Sets the name of the color.

    Returns:
        str: Foreground code.
    """
    return Colored(name).foreground()


def bg(name: int | str) -> str:
    """ This will be deprecated in the future, do not use with version >= 2.0.0,
    instead please use style() function (See issue #28).

    Args:
        name: Sets the name of the color.

    Returns:
        str: Background code.
    """
    return Colored(name).background()


def stylize(text: str, formatting: int | str, reset: bool = True) -> str:
    """ Conveniently styles your text as and resets ANSI codes at its end.

    Args:
        text: String type text.
        formatting: Sets the formatting (color or style) of the text.
        reset: Reset the formatting style at its end, default is True.

    Returns:
        str: Formatting string text.

    Example:
        >>> stylize('Hello, World!', fore('red'))
        '\x1b[31mHello, World!\x1b[0m'
        >>> stylize('Bold text', style('bold'), reset=False)
        '\x1b[1mBold text'
    """
    terminator: str = style('reset') if reset else ''
    return f'{"".join(str(formatting))}{text}{terminator}'


def _c0wrap(formatting: str) -> str:
    """Contribution by brrzap.
    Wrap a set of ANSI styles in C0 control codes for readline safety.

    Args:
        formatting (str): Sets the formatting (color or style) of the text.

    Returns:
        str: Code formatting.
    """
    C0_SOH: str = '\x01'  # mark the beginning of nonprinting characters
    C0_STX: str = '\x02'  # mark the end of nonprinting characters
    return f'{C0_SOH}{"".join(formatting)}{C0_STX}'


def stylize_interactive(text: str, formatting: str, reset: bool = True) -> str:
    """Contribution by: Jay Deiman, brrzap
    stylize() variant that adds C0 control codes (SOH/STX) for readline
    safety.

    Args:
        text (str): String type text.
        formatting (str): Sets the formatting (color or style) of the text.
        reset (bool, optional): Reset the formatting style at its end, default is True.

    Returns:
        str: Formatting string text.

    Example:
        # For use in interactive shells with readline, e.g., Python REPL
        >>> import readline
        >>> readline.set_pre_input_hook(lambda: print(stylize_interactive('Prompt:', fore('green')), end=''))
        >>> input()
        Prompt:user_input  # 'Prompt:' will appear green, input text will be default color
    """
    # problem: readline includes bare ANSI codes in width calculations.
    # solution: wrap nonprinting codes in SOH/STX when necessary.
    # see: https://gitlab.com/dslackw/colored/issues/5
    terminator: str = _c0wrap(style('reset')) if reset else ''
    return f'{_c0wrap(formatting)}{text}{terminator}'


def set_tty_aware(awareness: bool = True) -> None:
    """ Contribution by: Andreas Motl, Jay Deiman

    Makes all interactions here tty aware. This means that if either
    stdout or stderr are directed to something other than a tty,
    colorization will not be added.

    Args:
        awareness: Default is True.
    """
    # global TTY_AWARE
    # TTY_AWARE = awareness
    Config.TTY_AWARE = awareness


# Call this method once when the module is loaded to enable Windows virtual terminal processing.
# This prevents redundant API calls on every Colored object instantiation.
Colored.enable_windows_terminal_mode()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from typing import Optional

from .exceptions import InvalidControl
from .library import Library
from .utilities import Utilities

# pylint: disable=invalid-name


class Controls:  # pylint: disable=[R0903]

    """ Controls class managing the cursor moves.
    """

    def __init__(self) -> None:
        self._utils = Utilities()
        self._ESC: str = Library.ESC
        self._CONTROLS: dict[str, str] = Library.CONTROLS

    def nav(self, name: str, row: int, column: Optional[int] = None) -> str:
        """Args:
            name (str): Type of cursor move. (See library.py for available moves).
            row (int): Number of rows to move.
            column (Optional[int]): Number of columns to move. Required for 'position' move.

        Returns:
            str: An ANSI escape sequence string that performs the specified cursor movement.

        Examples:
            ESC[ row B: Cursor Down
            ESC[ row C: Cursor Forward
            ESC[ row D: Cursor Back
            ESC[ row E: Cursor Next Line
            ESC[ row F: Cursor Previous Line
            ESC[ row G: Cursor Horizontal Absolute
            ESC[ row; col; H: Cursor Position
            ESC[ row J: Erase in Display
            ESC[ row K: Erase in Line
            ESC[ row S: Scroll Up
            ESC[ row T: Scroll Down
        Raises:
             InvalidControl: Raises an Exception.
        """
        if not isinstance(name, str):
            raise TypeError(f"The 'name' argument must be a string, got {type(name).__name__}")
        self._utils.is_control_exist(name)

        if column is not None:
            if name.lower() == 'position':
                return f'{self._ESC}{row};{column}{self._CONTROLS[name.lower()]}'

            raise InvalidControl(f"InvalidControl: {name}: Expected 'position'")

        return f'{self._ESC}{row}{self._CONTROLS[name.lower()]}'



from colored.attributes import Style
from colored.colored import Colored
from colored.library import Library

ESC: str = '\x1b['
END: str = 'm'
UNDERLINE_COLOR: str = f'{ESC}4;58;5;'


def test_style_code() -> None:
    for code in Library.STYLES.values():
        style = Colored(code)
        assert f'{ESC}{code}{END}' == style.attribute()


def test_style_name() -> None:
    for name, code in Library.STYLES.items():
        style = Colored(name)
        assert f'{ESC}{code}{END}' == style.attribute()


def test_underline_color() -> None:
    for code in Library.COLORS.values():
        assert f'{UNDERLINE_COLOR}{code}{END}' == Style.underline_color(color=code)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

from typing import NoReturn

from .exceptions import InvalidColor
from .library import Library
from .utilities import Utilities


class MetaFore(type):
    """Metaclass to customize attribute access."""
    def __getattr__(cls, color: str) -> NoReturn:
        """Override __getattr__ to raise InvalidColor when an invalid color is accessed."""
        raise InvalidColor(f"InvalidColor: {color}")


class Fore(metaclass=MetaFore):

    """ Fore class contains predefined foreground colors for
    the command line (terminal).
    """

    _utils = Utilities()
    _END: str = Library.END
    _COLORS: dict[str, str] = Library.COLORS
    _FOREGROUND_256: str = Library.FOREGROUND_256
    _FOREGROUND_RGB: str = Library.FOREGROUND_RGB

    for _color, _code in _COLORS.items():
        vars()[_color] = f'{_FOREGROUND_256}{_code}{_END}'
        vars()[_color.upper()] = f'{_FOREGROUND_256}{_code}{_END}'

    @classmethod
    def rgb(cls: type[Fore], r: int | str, g: int | str, b: int | str) -> str:
        """Combination with text returns color text.

        Args:
            r (int | str): Sets the Red color.
            g (int | str): Sets the Green color.
            b (int | str): Sets the Blue color.

        Returns:
            str: Foreground RGB code.
        """
        r, g, b = cls._utils.is_percentage((r, g, b))
        return f'{cls._FOREGROUND_RGB}{r};{g};{b}{cls._END}'

    @classmethod
    def RGB(cls: type[Fore], r: int | str, g: int | str, b: int | str) -> str:  # pylint: disable=[C0103]
        """Combination with text returns color text.

        Args:
            r (int | str): Sets the Red color.
            g (int | str): Sets the Green color.
            b (int | str): Sets the Blue color.

        Returns:
            str: Foreground RGB code.
        """
        return cls.rgb(r, g, b)


class fore(Fore):  # pylint: disable=[C0103]
    """This will be deprecated in the future, do not use this for version >= 2.0.0,
        instead please use Fore class (See issue #28)."""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

from .exceptions import InvalidHexColor
from .library import Library


class Hex:
    """Provides methods for finding the nearest ANSI color code from a HEX color value."""

    def find(self, color: str) -> str:
        """ Contribution by Fredrik Klasson """

        if not color.startswith('#') or len(color) not in (4, 7):
            raise InvalidHexColor(f'InvalidHexColor: {color}')

        try:
            int(color[1:], 16)
        except ValueError as exc:
            raise InvalidHexColor(f'InvalidHexColor: {color}') from exc

        # Extend shorthand #ABC -> #AABBCC, like in CSS
        if len(color) == 4:
            color = '#' + color[1] * 2 + color[2] * 2 + color[3] * 2

        # Try an exact lookup, trying to favor lower numbers
        # (e.g. find 10 instead of 46 for #00FF00)
        for code, hex_color in Library.HEX_COLORS.items():
            if hex_color == color:
                return code

        # Try to find nearest match using a simple least squares fit.
        # We could try to factor in human perception bias by weighting
        # as suggested by <https://stackoverflow.com/a/1847112> but for
        # now lets just KISS and make upp our minds later, no?
        # (we do skip the sqrt since we just care for the relative value)

        # The reference color
        r, g, b = (int(color[1:3], 16), int(color[3:5], 16), int(color[5:7], 16))

        min_cube_d: int = self.square(0xFFFFFF)
        nearest: str = '15'

        for code, hex_color in Library.HEX_COLORS.items():
            cube_d: int = self.fit(hex_color[1:3], r) + self.fit(hex_color[3:5], g) + self.fit(hex_color[5:7], b)
            if cube_d < min_cube_d:
                min_cube_d = cube_d
                nearest = code

        return nearest

    @staticmethod
    def square(x: int) -> int:
        """Calculates the square of a given integer.

        Args:
            x (int): The integer number.

        Returns:
            int: The square of the input number.
        """
        return x * x

    def fit(self, hex_val: str, ref: int) -> int:
        """Calculates a component of the color difference for nearest match.

        This method converts a hexadecimal color component string to an integer,
        calculates the difference with a reference integer, and then applies
        the 'square' method to this difference. It's typically used as part of
        a least squares fit to find the closest color match.

        Args:
            hex_val (str): A string representing a two-digit hexadecimal color component (e.g., 'FF', '00').
            ref (int): An integer representing the reference color component (0-255).

        Returns:
            int: The result of applying the 'square' method to the difference
                 between the integer value of `hex_val` and `ref`.
        """
        return self.square(int(hex_val, 16) - ref)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .attributes import Style
from .background import Back
from .colored import (Colored, attr, back, back_rgb, bg, fg, fore, fore_rgb,
                      set_tty_aware, style, stylize, stylize_interactive)
from .controls import Controls
from .convert import Convert
from .cprint import cprint
from .foreground import Fore

__version__: str = '2.3.2'

__all__ = [
    'Colored', 'fore', 'back', 'style', 'fore_rgb', 'back_rgb',
    'fg', 'bg', 'attr', 'stylize', 'stylize_interactive', 'set_tty_aware',
    'cprint', 'Fore', 'Back', 'Style', 'Controls', 'Convert', '__version__'
]



import io
import sys

from colored.cprint import cprint, resolve_background, resolve_foreground
from colored.library import Library


def _capture(func: object, *args: object, **kwargs: object) -> str:
    old_stdout = sys.stdout
    sys.stdout = io.StringIO()
    func(*args, **kwargs)  # type: ignore[operator]
    output = sys.stdout.getvalue()
    sys.stdout = old_stdout
    return output


def test_cprint_fore_256() -> None:
    output = _capture(cprint, 'Hello', fore_256='red')
    assert 'Hello' in output


def test_cprint_back_256() -> None:
    output = _capture(cprint, 'Hello', back_256='blue')
    assert 'Hello' in output


def test_cprint_fore_rgb() -> None:
    output = _capture(cprint, 'Hello', fore_rgb=(255, 0, 0))
    assert 'Hello' in output


def test_cprint_back_rgb() -> None:
    output = _capture(cprint, 'Hello', back_rgb=(0, 255, 0))
    assert 'Hello' in output


def test_cprint_no_reset() -> None:
    output = _capture(cprint, 'Hello', fore_256='red', reset=False)
    assert 'Hello' in output


def test_cprint_formatting() -> None:
    output = _capture(cprint, 'Hello', formatting='bold')
    assert 'Hello' in output


def test_resolve_foreground_256() -> None:
    result = resolve_foreground('red', None)
    assert result != ''


def test_resolve_foreground_rgb() -> None:
    result = resolve_foreground(None, (255, 0, 0))
    assert result != ''


def test_resolve_foreground_empty() -> None:
    assert resolve_foreground(None, None) == ''


def test_resolve_background_256() -> None:
    result = resolve_background('red', None)
    assert result != ''


def test_resolve_background_rgb() -> None:
    result = resolve_background(None, (0, 255, 0))
    assert result != ''


def test_resolve_background_empty() -> None:
    assert resolve_background(None, None) == ''


def test_cprint_all_colors() -> None:
    for color in list(Library.COLORS.values())[:10]:
        output = _capture(cprint, 'Test', fore_256=color)
        assert 'Test' in output



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

from typing import Any, Optional

from .colored import Colored
from .colored import back_rgb as background_rgb
from .colored import fore_rgb as foreground_rgb


def cprint(text: str,  # pylint: disable=[R0913,R0917]
           fore_256: Optional[int | str] = None,
           back_256: Optional[int | str] = None,
           fore_rgb: Optional[tuple[int | str, int | str, int | str]] = None,
           back_rgb: Optional[tuple[int | str, int | str, int | str]] = None,
           formatting: int | str = '',
           line_color: int | str = '',
           reset: bool = True,
           **kwargs: Any) -> None:  # noqa: ANN401
    """Looks like a patch to a built-in python print() function that allows
    to pass colored text and style, to this function.

    Args:
        text (str): String type text.
        fore_256 (Optional[int | str], optional): Sets the foreground color.
        back_256 (Optional[int | str], optional): Sets the background color.
        fore_rgb (Optional[tuple], optional): Sets the foreground color.
        back_rgb (Optional[tuple], optional): Sets the background color.
        formatting (int | str, optional): Sets the style of the text.
        line_color (int | str, optional): Sets the underline color.
        reset (bool, optional): Reset the formatting style after print, default is True.
        **kwargs (dict[str, Any]): Description
    """
    styling = Colored(formatting).attribute(line_color)
    terminator = Colored('reset').attribute() if reset else ''

    fore_color = resolve_foreground(fore_256, fore_rgb)
    back_color = resolve_background(back_256, back_rgb)

    print(f'{styling}{back_color}{fore_color}{text}{terminator}', **kwargs)


def resolve_foreground(fore_256: Optional[int | str], fore_rgb: Optional[tuple[int | str, int | str, int | str]]) -> str:
    """Returns the correct foreground color string.

    Args:
        fore_256 (Optional[int | str]): Description
        fore_rgb (Optional[tuple]): Description

    Returns:
        str: Foreground color code.
    """
    fore_color: str = ''
    if fore_256:
        fore_color = Colored(fore_256).foreground()
    elif fore_rgb:
        fr, fg, fb = fore_rgb
        fore_color = foreground_rgb(fr, fg, fb)

    return fore_color


def resolve_background(back_256: Optional[int | str], back_rgb: Optional[tuple[int | str, int | str, int | str]]) -> str:
    """Returns the correct background color string.

    Args:
        back_256 (Optional[int | str]): Description
        back_rgb (Optional[tuple]): Description

    Returns:
        str: Background color code.
    """
    back_color: str = ''
    if back_256:
        back_color = Colored(back_256).background()
    elif back_rgb:
        br, bg, bb = back_rgb
        back_color = background_rgb(br, bg, bb)

    return back_color



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from colored import fore, style
from colored.convert import Convert
from colored.library import Library


def main() -> None:
    """Print all codes and names side by side.
    """
    convert = Convert()
    for hex_code, (color, ansi_code) in zip(Library.HEX_COLORS.values(), Library.COLORS.items()):
        print(f'{fore(ansi_code)}{color:<19}{style(0)} {repr(convert.hex_to_ansi(hex_code)):<16}'
              f'{repr(convert.ansi_to_hex(int(ansi_code))):>9}')


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import time
from random import randint

from colored import back, fore, style
from colored.hexadecimal import Hex
from colored.library import Library


def compare_with_expected(in_hex: str, expected: str) -> bool:
    """ Contribution by Fredrik Klasson. """
    hexad = Hex()
    nearest = hexad.find(in_hex)
    # Look up the matched hex value (e.g. xterm colors 10 and 46 are
    # the same hex value)
    match = Library.HEX_COLORS[nearest] == Library.HEX_COLORS[expected]

    e_str = '%s%s##%s' % (fore(expected), back(expected), style('reset'))
    n_str = '%s%s##%s' % (fore(nearest), back(nearest), style('reset'))
    print("%s: %s => %s = %s" % ('pass' if match else 'FAIL', in_hex, n_str, e_str))

    return match


def main() -> None:
    """Test the nearest matching hex color lookup.
    Contribution by Fredrik Klasson. """
    print('            Nearest   Expected')

    test_set = {
        '1': ('#7f0000', '#800000', '#810000'),
        '2': ('#007f00', '#008000', '#008100'),
        '4': ('#00007f', '#000080', '#000081'),
        '10': ('#00fe00', '#00ff00', '#01ff00'),
        '11': ('#fe0000', '#ff0000', '#ff0100'),
        '12': ('#0000fe', '#0000ff', '#0100ff'),
        '13': ('#feff00', '#ffff00', '#fffe00'),
        '14': ('#00feff', '#00ffff', '#00fffe'),
        '15': ('#ff00fe', '#ff00ff', '#ff01ff'),
        '16': ('#fffeff', '#ffffff', '#effeff'),
        '17': ('#010101', '#000000', '#010000'),
        '18': ('#1e346c', '#00005f', '#1e346c'),
    }

    all_ok = True
    for expected, color_set in test_set.items():
        for hex_code in color_set:
            if not compare_with_expected(hex_code, expected):
                all_ok = False

    print('-' * 78)

    try:
        T_sta = time.perf_counter()
        r_row = ''
        g_row = ''
        b_row = ''
        i_row = ''
        hexad = Hex()  # Instantiate Hex once here

        for c in range(0, 0xFF + 1, 16):
            # Red gradient
            hex_code = '#%02x0000' % (c,)
            nearest_ansi_r = hexad.find(hex_code)  # Get the ANSI code
            r_row += '%s%s#' % (fore(nearest_ansi_r), back(nearest_ansi_r))

            # Green gradient
            hex_code = '#00%02x00' % (c,)
            nearest_ansi_g = hexad.find(hex_code)  # Get the ANSI code
            g_row += '%s%s#' % (fore(nearest_ansi_g), back(nearest_ansi_g))

            # Blue gradient
            hex_code = '#0000%02x' % (c,)
            nearest_ansi_b = hexad.find(hex_code)  # Get the ANSI code
            b_row += '%s%s#' % (fore(nearest_ansi_b), back(nearest_ansi_b))

            # Grayscale gradient
            hex_code = '#' + ('%02x' % (c,)) * 3
            nearest_ansi_i = hexad.find(hex_code)  # Get the ANSI code
            i_row += '%s%s#' % (fore(nearest_ansi_i), back(nearest_ansi_i))

        print('%s%s %s%s %s%s %s%s' % (
            r_row, style('reset'), g_row, style('reset'), b_row, style('reset'), i_row, style('reset')))
        dT = time.perf_counter() - T_sta
        print('Lookup time: %0.4f s => %0.4f s/lookup' % (dT, dT / (2 * 4 * 0xFF)))
        print('-' * 78)
    except Exception as e:  # pylint: disable=[W0718]
        print('Whopsie, something %s-ish went wrong: %s' % (e.__class__.__name__, e))
        import traceback  # pylint: disable=[C0415]
        traceback.print_exc()
        all_ok = False

    # This is just for fun, almost... let's call it a
    # "non-deterministic check that it doesn't throw any exceptions"
    try:
        T_sta = time.perf_counter()
        hexad = Hex() # Re-instantiate Hex for this section
        for y in range(0, 20):
            for x in range(0, 30):
                rnd = randint(0, 0xffffff)
                hex_code = '#%06x' % (rnd,)
                hexinv = '#%06x' % (0xffffff - rnd,)

                # Get nearest ANSI codes for the random hex values
                nearest_hexinv_ansi = hexad.find(hexinv)
                nearest_hex_code_ansi = hexad.find(hex_code)

                print('%s%s::' % (fore(nearest_hexinv_ansi), back(nearest_hex_code_ansi)), end='')
            print('')
        print(style('reset'))
        dT = time.perf_counter() - T_sta
        print('Random test time: %0.4f s => %0.4f s/lookup' % (dT, dT / (30 * 20 * 2)))
        print('-' * 78)
    except Exception as e:  # pylint: disable=[W0718]
        print('Whopsie, something %s-ish went wrong: %s' % (e.__class__.__name__, e))
        import traceback
        traceback.print_exc()
        all_ok = False

    if not all_ok:
        sys.exit(1)


if __name__ == "__main__":
    main()



import pytest

from colored.colored import (Colored, attr, back_rgb, bg, fg, fore_rgb, set_tty_aware,
                              style, stylize, stylize_interactive)
from colored.exceptions import InvalidColor, InvalidHexColor, InvalidStyle
from colored.library import Library

ESC: str = '\x1b['
FOREGROUND_RGB: str = f'{ESC}38;2;'
BACKGROUND_RGB: str = f'{ESC}48;2;'
END: str = 'm'


def test_style_func() -> None:
    assert style('bold') == f'{ESC}1{END}'


def test_style_func_with_underline_color() -> None:
    for code in Library.COLORS.values():
        result = style('underline', code)
        assert result != ''


def test_fore_rgb_func() -> None:
    assert fore_rgb(255, 0, 0) == f'{FOREGROUND_RGB}255;0;0{END}'


def test_back_rgb_func() -> None:
    assert back_rgb(0, 255, 0) == f'{BACKGROUND_RGB}0;255;0{END}'


def test_fore_rgb_percentages() -> None:
    result = fore_rgb('100%', '0%', '0%')
    assert FOREGROUND_RGB in result


def test_back_rgb_percentages() -> None:
    result = back_rgb('0%', '100%', '0%')
    assert BACKGROUND_RGB in result


def test_fore_rgb_clamping() -> None:
    assert fore_rgb(300, -1, 128) == f'{FOREGROUND_RGB}255;0;128{END}'


def test_attr_func() -> None:
    assert attr('bold') == f'{ESC}1{END}'


def test_fg_func() -> None:
    for name, code in Library.COLORS.items():
        assert fg(name) == f'{ESC}38;5;{code}{END}'


def test_bg_func() -> None:
    for name, code in Library.COLORS.items():
        assert bg(name) == f'{ESC}48;5;{code}{END}'


def test_stylize() -> None:
    result = stylize('Hello', fore_rgb(255, 0, 0))
    assert 'Hello' in result
    assert style('reset') in result


def test_stylize_no_reset() -> None:
    result = stylize('Hello', fore_rgb(255, 0, 0), reset=False)
    assert 'Hello' in result
    assert style('reset') not in result


def test_stylize_interactive() -> None:
    result = stylize_interactive('Hello', fore_rgb(255, 0, 0))
    assert 'Hello' in result
    assert '\x01' in result
    assert '\x02' in result


def test_hex_foreground() -> None:
    result = Colored('#ff0000').foreground()
    assert result != ''
    assert FOREGROUND_RGB not in result or ESC in result


def test_hex_background() -> None:
    result = Colored('#00ff00').background()
    assert result != ''


def test_set_tty_aware() -> None:
    set_tty_aware(False)
    assert Colored('red').foreground() != ''
    set_tty_aware(True)


def test_force_color_disabled(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setenv('FORCE_COLOR', '0')
    assert Colored('red').foreground() == ''


def test_no_color_disables(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.delenv('FORCE_COLOR', raising=False)
    monkeypatch.setenv('NO_COLOR', '1')
    assert Colored('red').foreground() == ''


def test_invalid_color_raises() -> None:
    with pytest.raises(InvalidColor):
        Colored('not_a_color').foreground()


def test_invalid_hex_color_raises() -> None:
    with pytest.raises(InvalidHexColor):
        Colored('#gggggg').foreground()


def test_invalid_style_raises() -> None:
    with pytest.raises(InvalidStyle):
        Colored('not_a_style').attribute()



import unittest

from colored.attributes import Style
from colored.colored import Colored
from colored.library import Library

# pylint: disable=invalid-name


class TestStyles(unittest.TestCase):

    """Test for styles attributes names and codes.
    """

    def setUp(self) -> None:
        self.ESC: str = '\x1b['
        self.END: str = 'm'
        self.UNDERLINE_COLOR: str = f'{self.ESC}4;58;5;'

    def test_style_code(self) -> None:
        """Test styles attributes by code.
        """
        for code in Library.STYLES.values():
            style = Colored(code)
            self.assertEqual(f'{self.ESC}{code}{self.END}', style.attribute())

    def test_style_name(self) -> None:
        """Test styles attributes by name.
        """
        for name, code in Library.STYLES.items():
            style = Colored(name)
            self.assertEqual(f'{self.ESC}{code}{self.END}', style.attribute())

    def test_underline_color(self) -> None:
        """Test underline attribute.
        """
        for code in Library.COLORS.values():
            self.assertEqual(f'{self.UNDERLINE_COLOR}{code}{self.END}', Style.underline_color(color=code))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class InvalidColor(Exception):
    """Custom Exception for invalid colors."""

    def __init__(self, message: str) -> None:
        super().__init__(message)


class InvalidHexColor(Exception):
    """ Custom Exception for invalid hex colors. """

    def __init__(self, message: str) -> None:
        super().__init__(message)


class InvalidStyle(Exception):
    """ Custom Exception for invalid style. """

    def __init__(self, message: str) -> None:
        super().__init__(message)


class InvalidControl(Exception):
    """ Custom Exception for invalid navigation. """

    def __init__(self, message: str) -> None:
        super().__init__(message)



from colored.colored import Colored, fore, fore_rgb
from colored.foreground import Fore
from colored.library import Library

ESC: str = '\x1b['
FOREGROUND_256: str = f'{ESC}38;5;'
FOREGROUND_RGB: str = f'{ESC}38;2;'
END: str = 'm'


def test_foreground_256_code() -> None:
    for code in Library.COLORS.values():
        color = Colored(code)
        assert f'{FOREGROUND_256}{code}{END}' == color.foreground()


def test_foreground_256_name() -> None:
    for name, code in Library.COLORS.items():
        color = Colored(name)
        assert f'{FOREGROUND_256}{code}{END}' == color.foreground()


def test_fore_256_func() -> None:
    for name, code in Library.COLORS.items():
        color = Colored(name)
        assert fore(code) == color.foreground()


def test_foreground_rgb() -> None:
    for code in Library.COLORS.values():
        assert f'{FOREGROUND_RGB}{code};{code};{code}{END}' == Fore.rgb(code, code, code)


def test_foreground_rgb_func() -> None:
    for code in Library.COLORS.values():
        assert f'{FOREGROUND_RGB}{code};{code};{code}{END}' == fore_rgb(code, code, code)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from colored import Back, Fore, Style
from colored.library import Library


def main() -> None:
    """Test foreground and background colors.
    """

    for color in Library.COLORS:
        print(f"{getattr(Fore, color)} This is the color {color} {Style.RESET}")

    for color in Library.COLORS:
        print(f"{getattr(Back, color)} This is the color {color:>20} {Style.RESET}")


if __name__ == '__main__':
    main()



from colored.controls import Controls
from colored.exceptions import InvalidControl


ESC: str = '\x1b['


def test_nav_cursor_down() -> None:
    assert Controls().nav('down', 3) == f'{ESC}3B'


def test_nav_cursor_up() -> None:
    assert Controls().nav('up', 2) == f'{ESC}2A'


def test_nav_position() -> None:
    assert Controls().nav('position', 5, 10) == f'{ESC}5;10H'


def test_nav_position_column_zero() -> None:
    assert Controls().nav('position', 5, 0) == f'{ESC}5;0H'


def test_nav_invalid_column_raises() -> None:
    try:
        Controls().nav('down', 3, 5)
    except InvalidControl:
        pass


def test_nav_invalid_name_raises() -> None:
    try:
        Controls().nav('invalid', 1)
    except InvalidControl:
        pass



import pytest

from colored.exceptions import InvalidColor, InvalidControl, InvalidHexColor, InvalidStyle


def test_exception_raised_invalidcolor() -> None:
    with pytest.raises(InvalidColor) as ctx:
        raise InvalidColor("InvalidColor: red_90")
    assert str(ctx.value) == "InvalidColor: red_90"


def test_exception_message_invalidcolor() -> None:
    assert str(InvalidColor("InvalidColor: red_90")) == "InvalidColor: red_90"


def test_exception_inheritance_invalidcolor() -> None:
    assert isinstance(InvalidColor("Test"), Exception)


def test_exception_raised_invalidhexcolor() -> None:
    with pytest.raises(InvalidHexColor) as ctx:
        raise InvalidHexColor("InvalidHexColor: #005fdj")
    assert str(ctx.value) == "InvalidHexColor: #005fdj"


def test_exception_message_invalidhexcolor() -> None:
    assert str(InvalidHexColor("InvalidHexColor: blue_15")) == "InvalidHexColor: blue_15"


def test_exception_inheritance_invalidhexcolor() -> None:
    assert isinstance(InvalidHexColor("Test"), Exception)


def test_exception_raised_invalidcontrol() -> None:
    with pytest.raises(InvalidControl) as ctx:
        raise InvalidControl("InvalidControl: upper")
    assert str(ctx.value) == "InvalidControl: upper"


def test_exception_message_invalidcontrol() -> None:
    assert str(InvalidControl("InvalidControl: righting")) == "InvalidControl: righting"


def test_exception_inheritance_invalidcontrol() -> None:
    assert isinstance(InvalidControl("Test"), Exception)


def test_exception_raised_invalidstyle() -> None:
    with pytest.raises(InvalidStyle) as ctx:
        raise InvalidStyle("InvalidStyle: bolb")
    assert str(ctx.value) == "InvalidStyle: bolb"


def test_exception_message_invalidstyle() -> None:
    assert str(InvalidStyle("InvalidStyle: bolb")) == "InvalidStyle: bolb"


def test_exception_inheritance_invalidstyle() -> None:
    assert isinstance(InvalidStyle("Test"), Exception)
